/*
This file is part of the Juju GUI, which lets users view and manage Juju
environments within a graphical interface (https://launchpad.net/juju-gui).
Copyright (C) 2017 Canonical Ltd.

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU Affero General Public License version 3, as published by
the Free Software Foundation.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero
General Public License for more details.

You should have received a copy of the GNU Affero General Public License along
with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

'use strict';

var juju = { components: {} }; // eslint-disable-line no-unused-vars

chai.config.includeStack = true;
chai.config.truncateThreshold = 0;

describe('DeploymentPayment', function () {
  let acl, addressFields, getUser, user;

  beforeAll(function (done) {
    // By loading this file it adds the component to the juju components.
    YUI().use('deployment-payment', function () {
      done();
    });
  });

  beforeEach(() => {
    acl = { isReadOnly: sinon.stub().returns(false) };
    user = {
      paymentMethods: [{
        name: 'Company'
      }]
    };
    getUser = sinon.stub().callsArgWith(1, null, {
      paymentMethods: [{
        name: 'Company'
      }]
    });
    addressFields = React.createElement(
      'div',
      null,
      React.createElement(juju.components.InsetSelect, {
        disabled: false,
        label: 'Country',
        onChange: null,
        options: [] }),
      React.createElement(juju.components.GenericInput, {
        disabled: false,
        label: 'Full name',
        required: true }),
      React.createElement(juju.components.GenericInput, {
        disabled: false,
        label: 'Address line 1',
        required: true }),
      React.createElement(juju.components.GenericInput, {
        disabled: false,
        label: 'Address line 2 (optional)',
        required: false }),
      React.createElement(juju.components.GenericInput, {
        disabled: false,
        label: 'State/province (optional)',
        required: false }),
      React.createElement(
        'div',
        { className: 'twelve-col' },
        React.createElement(
          'div',
          { className: 'six-col' },
          React.createElement(juju.components.GenericInput, {
            disabled: false,
            label: 'Town/city',
            required: true })
        ),
        React.createElement(
          'div',
          { className: 'six-col last-col' },
          React.createElement(juju.components.GenericInput, {
            disabled: false,
            label: 'Postcode',
            required: true })
        ),
        React.createElement(
          'div',
          { className: 'four-col' },
          React.createElement(juju.components.InsetSelect, {
            disabled: false,
            label: 'Country code',
            onChange: null,
            options: [] })
        ),
        React.createElement(
          'div',
          { className: 'eight-col last-col' },
          React.createElement(juju.components.GenericInput, {
            disabled: false,
            label: 'Phone number',
            required: true })
        )
      )
    );
  });

  it('can display a loading spinner', function () {
    const renderer = jsTestUtils.shallowRender(React.createElement(juju.components.DeploymentPayment, {
      acl: acl,
      addNotification: sinon.stub(),
      getUser: sinon.stub(),
      setPaymentUser: sinon.stub(),
      username: 'spinach' }), true);
    const output = renderer.getRenderOutput();
    const expected = React.createElement(
      'div',
      { className: 'deployment-payment' },
      React.createElement(juju.components.Spinner, null)
    );
    assert.deepEqual(output, expected);
  });

  it('can store the user details', function () {
    const setPaymentUser = sinon.stub();
    jsTestUtils.shallowRender(React.createElement(juju.components.DeploymentPayment, {
      acl: acl,
      addNotification: sinon.stub(),
      getUser: getUser,
      setPaymentUser: setPaymentUser,
      username: 'spinach' }));
    assert.equal(setPaymentUser.callCount, 1);
    assert.deepEqual(setPaymentUser.args[0][0], user);
  });

  it('can display payment methods', function () {
    const renderer = jsTestUtils.shallowRender(React.createElement(juju.components.DeploymentPayment, {
      acl: acl,
      addNotification: sinon.stub(),
      getUser: getUser,
      paymentUser: user,
      setPaymentUser: sinon.stub(),
      username: 'spinach' }), true);
    const output = renderer.getRenderOutput();
    const expected = React.createElement(
      'div',
      { className: 'deployment-payment' },
      React.createElement(
        'ul',
        { className: 'deployment-payment__methods twelve-col' },
        [React.createElement(
          'li',
          { className: 'deployment-payment__method',
            key: 'Company0' },
          React.createElement(juju.components.AccountPaymentMethodCard, {
            card: { name: 'Company' } })
        )]
      )
    );
    assert.deepEqual(output, expected);
  });

  it('can display an error when getting users', function () {
    getUser = sinon.stub().callsArgWith(1, 'failed', null);
    const addNotification = sinon.stub();
    const setPaymentUser = sinon.stub();
    jsTestUtils.shallowRender(React.createElement(juju.components.DeploymentPayment, {
      acl: acl,
      addNotification: addNotification,
      getUser: getUser,
      setPaymentUser: setPaymentUser,
      username: 'spinach' }));
    assert.equal(addNotification.callCount, 1);
    assert.deepEqual(addNotification.args[0][0], {
      title: 'Could not load user info',
      message: 'Could not load user info: failed',
      level: 'error'
    });
  });

  it('can display a personal form', function () {
    const renderer = jsTestUtils.shallowRender(React.createElement(juju.components.DeploymentPayment, {
      acl: acl,
      addNotification: sinon.stub(),
      getUser: getUser,
      paymentUser: null,
      setPaymentUser: sinon.stub(),
      username: 'spinach' }), true);
    const instance = renderer.getMountedInstance();
    const output = renderer.getRenderOutput();
    const options = output.props.children.props.children[0].props.children[0].props.children;
    const expected = React.createElement(
      'div',
      { className: 'deployment-payment' },
      React.createElement(
        'form',
        { className: 'deployment-payment__form' },
        React.createElement(
          'div',
          { className: 'deployment-payment__form-content' },
          React.createElement(
            'ul',
            { className: 'deployment-payment__form-type' },
            React.createElement(
              'li',
              { className: 'deployment-payment__form-type-option' },
              React.createElement(
                'label',
                { htmlFor: 'personal' },
                React.createElement('input', { checked: true,
                  id: 'personal',
                  name: 'formType',
                  onChange: options[0].props.children.props.children[0].props.onChange,
                  type: 'radio' }),
                'Personal use'
              )
            ),
            React.createElement(
              'li',
              { className: 'deployment-payment__form-type-option' },
              React.createElement(
                'label',
                { htmlFor: 'business' },
                React.createElement('input', { checked: false,
                  id: 'business',
                  name: 'formType',
                  onChange: options[1].props.children.props.children[0].props.onChange,
                  type: 'radio' }),
                'Business use'
              )
            )
          ),
          null,
          React.createElement(
            'h2',
            { className: 'deployment-payment__title' },
            'Name and address'
          ),
          null,
          addressFields,
          React.createElement(
            'h2',
            { className: 'deployment-payment__title' },
            'Payment information'
          ),
          React.createElement(juju.components.GenericInput, {
            disabled: false,
            label: 'Card number',
            required: true }),
          React.createElement(
            'div',
            { className: 'twelve-col' },
            React.createElement(
              'div',
              { className: 'six-col' },
              React.createElement(juju.components.GenericInput, {
                disabled: false,
                label: 'Expiry MM/YY',
                required: true })
            ),
            React.createElement(
              'div',
              { className: 'six-col last-col' },
              React.createElement(juju.components.GenericInput, {
                disabled: false,
                label: 'Security number (CVC)',
                required: true })
            )
          ),
          React.createElement(
            'div',
            { className: 'twelve-col' },
            React.createElement(juju.components.GenericInput, {
              disabled: false,
              label: 'Name on card',
              required: true })
          ),
          React.createElement(
            'label',
            { htmlFor: 'cardAddressSame' },
            React.createElement('input', { checked: true,
              id: 'cardAddressSame',
              name: 'cardAddressSame',
              onChange: instance._handleCardSameChange,
              type: 'checkbox' }),
            'Credit or debit card address is the same as above'
          ),
          React.createElement(
            'label',
            { htmlFor: 'billingAddressSame' },
            React.createElement('input', { checked: true,
              id: 'billingAddressSame',
              name: 'billingAddressSame',
              onChange: instance._handleBillingSameChange,
              type: 'checkbox' }),
            'Billing address is the same as above'
          ),
          null,
          null
        ),
        React.createElement(
          'div',
          { className: 'deployment-payment__add' },
          React.createElement(juju.components.GenericButton, {
            action: null,
            disabled: false,
            type: 'inline-neutral',
            title: 'Add payment details' })
        )
      )
    );
    assert.deepEqual(output, expected);
  });

  it('can display a business form', function () {
    const renderer = jsTestUtils.shallowRender(React.createElement(juju.components.DeploymentPayment, {
      acl: acl,
      addNotification: sinon.stub(),
      getUser: getUser,
      paymentUser: null,
      setPaymentUser: sinon.stub(),
      username: 'spinach' }), true);
    const instance = renderer.getMountedInstance();
    let output = renderer.getRenderOutput();
    output.props.children.props.children[0].props.children[0].props.children[1].props.children.props.children[0].props.onChange();
    output = renderer.getRenderOutput();
    const options = output.props.children.props.children[0].props.children[0].props.children;
    const expected = React.createElement(
      'div',
      { className: 'deployment-payment' },
      React.createElement(
        'form',
        { className: 'deployment-payment__form' },
        React.createElement(
          'div',
          { className: 'deployment-payment__form-content' },
          React.createElement(
            'ul',
            { className: 'deployment-payment__form-type' },
            React.createElement(
              'li',
              { className: 'deployment-payment__form-type-option' },
              React.createElement(
                'label',
                { htmlFor: 'personal' },
                React.createElement('input', { checked: false,
                  id: 'personal',
                  name: 'formType',
                  onChange: options[0].props.children.props.children[0].props.onChange,
                  type: 'radio' }),
                'Personal use'
              )
            ),
            React.createElement(
              'li',
              { className: 'deployment-payment__form-type-option' },
              React.createElement(
                'label',
                { htmlFor: 'business' },
                React.createElement('input', { checked: true,
                  id: 'business',
                  name: 'formType',
                  onChange: options[1].props.children.props.children[0].props.onChange,
                  type: 'radio' }),
                'Business use'
              )
            )
          ),
          React.createElement(
            'div',
            { className: 'deployment-payment__vat' },
            React.createElement(juju.components.GenericInput, {
              disabled: false,
              label: 'VAT number (optional)',
              required: false })
          ),
          React.createElement(
            'h2',
            { className: 'deployment-payment__title' },
            'Name and address'
          ),
          React.createElement(juju.components.GenericInput, {
            disabled: false,
            label: 'Business name',
            required: true }),
          addressFields,
          React.createElement(
            'h2',
            { className: 'deployment-payment__title' },
            'Payment information'
          ),
          React.createElement(juju.components.GenericInput, {
            disabled: false,
            label: 'Card number',
            required: true }),
          React.createElement(
            'div',
            { className: 'twelve-col' },
            React.createElement(
              'div',
              { className: 'six-col' },
              React.createElement(juju.components.GenericInput, {
                disabled: false,
                label: 'Expiry MM/YY',
                required: true })
            ),
            React.createElement(
              'div',
              { className: 'six-col last-col' },
              React.createElement(juju.components.GenericInput, {
                disabled: false,
                label: 'Security number (CVC)',
                required: true })
            )
          ),
          React.createElement(
            'div',
            { className: 'twelve-col' },
            React.createElement(juju.components.GenericInput, {
              disabled: false,
              label: 'Name on card',
              required: true })
          ),
          React.createElement(
            'label',
            { htmlFor: 'cardAddressSame' },
            React.createElement('input', { checked: true,
              id: 'cardAddressSame',
              name: 'cardAddressSame',
              onChange: instance._handleCardSameChange,
              type: 'checkbox' }),
            'Credit or debit card address is the same as above'
          ),
          React.createElement(
            'label',
            { htmlFor: 'billingAddressSame' },
            React.createElement('input', { checked: true,
              id: 'billingAddressSame',
              name: 'billingAddressSame',
              onChange: instance._handleBillingSameChange,
              type: 'checkbox' }),
            'Billing address is the same as above'
          ),
          null,
          null
        ),
        React.createElement(
          'div',
          { className: 'deployment-payment__add' },
          React.createElement(juju.components.GenericButton, {
            action: null,
            disabled: false,
            type: 'inline-neutral',
            title: 'Add payment details' })
        )
      )
    );
    assert.deepEqual(output, expected);
  });

  it('can display card and billing address fields', function () {
    const renderer = jsTestUtils.shallowRender(React.createElement(juju.components.DeploymentPayment, {
      acl: acl,
      addNotification: sinon.stub(),
      getUser: getUser,
      paymentUser: null,
      setPaymentUser: sinon.stub(),
      username: 'spinach' }), true);
    const instance = renderer.getMountedInstance();
    let output = renderer.getRenderOutput();
    const formContent = output.props.children.props.children[0].props.children;
    formContent[9].props.children[0].props.onChange({ currentTarget: { checked: false } });
    formContent[10].props.children[0].props.onChange({ currentTarget: { checked: false } });
    output = renderer.getRenderOutput();
    const options = output.props.children.props.children[0].props.children[0].props.children;
    const expected = React.createElement(
      'div',
      { className: 'deployment-payment' },
      React.createElement(
        'form',
        { className: 'deployment-payment__form' },
        React.createElement(
          'div',
          { className: 'deployment-payment__form-content' },
          React.createElement(
            'ul',
            { className: 'deployment-payment__form-type' },
            React.createElement(
              'li',
              { className: 'deployment-payment__form-type-option' },
              React.createElement(
                'label',
                { htmlFor: 'personal' },
                React.createElement('input', { checked: true,
                  id: 'personal',
                  name: 'formType',
                  onChange: options[0].props.children.props.children[0].props.onChange,
                  type: 'radio' }),
                'Personal use'
              )
            ),
            React.createElement(
              'li',
              { className: 'deployment-payment__form-type-option' },
              React.createElement(
                'label',
                { htmlFor: 'business' },
                React.createElement('input', { checked: false,
                  id: 'business',
                  name: 'formType',
                  onChange: options[1].props.children.props.children[0].props.onChange,
                  type: 'radio' }),
                'Business use'
              )
            )
          ),
          null,
          React.createElement(
            'h2',
            { className: 'deployment-payment__title' },
            'Name and address'
          ),
          null,
          addressFields,
          React.createElement(
            'h2',
            { className: 'deployment-payment__title' },
            'Payment information'
          ),
          React.createElement(juju.components.GenericInput, {
            disabled: false,
            label: 'Card number',
            required: true }),
          React.createElement(
            'div',
            { className: 'twelve-col' },
            React.createElement(
              'div',
              { className: 'six-col' },
              React.createElement(juju.components.GenericInput, {
                disabled: false,
                label: 'Expiry MM/YY',
                required: true })
            ),
            React.createElement(
              'div',
              { className: 'six-col last-col' },
              React.createElement(juju.components.GenericInput, {
                disabled: false,
                label: 'Security number (CVC)',
                required: true })
            )
          ),
          React.createElement(
            'div',
            { className: 'twelve-col' },
            React.createElement(juju.components.GenericInput, {
              disabled: false,
              label: 'Name on card',
              required: true })
          ),
          React.createElement(
            'label',
            { htmlFor: 'cardAddressSame' },
            React.createElement('input', { checked: false,
              id: 'cardAddressSame',
              name: 'cardAddressSame',
              onChange: instance._handleCardSameChange,
              type: 'checkbox' }),
            'Credit or debit card address is the same as above'
          ),
          React.createElement(
            'label',
            { htmlFor: 'billingAddressSame' },
            React.createElement('input', { checked: false,
              id: 'billingAddressSame',
              name: 'billingAddressSame',
              onChange: instance._handleBillingSameChange,
              type: 'checkbox' }),
            'Billing address is the same as above'
          ),
          React.createElement(
            'div',
            null,
            React.createElement(
              'h2',
              { className: 'deployment-payment__title' },
              'Card address'
            ),
            addressFields
          ),
          React.createElement(
            'div',
            null,
            React.createElement(
              'h2',
              { className: 'deployment-payment__title' },
              'Billing address'
            ),
            addressFields
          )
        ),
        React.createElement(
          'div',
          { className: 'deployment-payment__add' },
          React.createElement(juju.components.GenericButton, {
            action: null,
            disabled: false,
            type: 'inline-neutral',
            title: 'Add payment details' })
        )
      )
    );
    assert.deepEqual(output, expected);
  });

  it('can abort requests when unmounting', function () {
    const abort = sinon.stub();
    getUser = sinon.stub().returns({ abort: abort });
    const component = jsTestUtils.shallowRender(React.createElement(juju.components.DeploymentPayment, {
      acl: acl,
      addNotification: sinon.stub(),
      getUser: getUser,
      setPaymentUser: sinon.stub(),
      username: 'spinach' }), true);
    component.unmount();
    assert.equal(abort.callCount, 1);
  });
});