/*
This file is part of the Juju GUI, which lets users view and manage Juju
environments within a graphical interface (https://launchpad.net/juju-gui).
Copyright (C) 2017 Canonical Ltd.

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU Affero General Public License version 3, as published by
the Free Software Foundation.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero
General Public License for more details.

You should have received a copy of the GNU Affero General Public License along
with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

'use strict';

var juju = { components: {} }; // eslint-disable-line no-unused-vars

chai.config.includeStack = true;
chai.config.truncateThreshold = 0;

describe('Sharing', () => {
  const humanizeTimestamp = sinon.stub().returns('9 minutes ago');
  let users;

  beforeAll(done => {
    // By loading this file it adds the component to the juju components.
    YUI().use('sharing', () => {
      done();
    });
  });

  beforeEach(() => {
    users = [{
      name: 'drwho@external',
      displayName: 'drwho',
      domain: 'Ubuntu SSO',
      lastConnection: '9 minutes ago',
      access: 'admin'
    }, {
      name: 'rose',
      displayName: 'Rose',
      domain: 'local',
      lastConnection: null,
      access: 'write'
    }, {
      name: 'dalek',
      displayName: 'Dalek',
      lastConnection: null,
      access: 'write',
      err: 'exterminate!'
    }];
  });

  it('can render with no users', () => {
    const getModelUserInfo = sinon.stub().callsArgWith(0, null, []);
    const closeHandler = sinon.stub();
    const renderer = jsTestUtils.shallowRender(React.createElement(juju.components.Sharing, {
      canShareModel: false,
      closeHandler: closeHandler,
      getModelUserInfo: getModelUserInfo,
      grantModelAccess: sinon.stub(),
      humanizeTimestamp: humanizeTimestamp,
      revokeModelAccess: sinon.stub() }), true);
    const output = renderer.getRenderOutput();
    const expected = React.createElement(
      juju.components.Popup,
      {
        className: 'sharing__popup',
        close: closeHandler,
        title: 'Share' },
      undefined,
      React.createElement(
        'div',
        { className: 'sharing__users-header' },
        React.createElement(
          'div',
          { className: 'sharing__users-header-user' },
          'User'
        ),
        React.createElement(
          'div',
          { className: 'sharing__users-header-access' },
          'Access'
        )
      ),
      React.createElement(
        'div',
        { className: 'sharing__users' },
        undefined
      ),
      React.createElement(juju.components.GenericButton, {
        title: 'Done',
        action: closeHandler,
        type: 'inline-neutral',
        extraClasses: 'right'
      })
    );
    assert.deepEqual(output, expected);
  });

  it('can render with a spinner', () => {
    const getModelUserInfo = sinon.stub().callsArgWith(0, null, []);
    const renderer = jsTestUtils.shallowRender(React.createElement(juju.components.Sharing, {
      canShareModel: false,
      getModelUserInfo: getModelUserInfo,
      grantModelAccess: sinon.stub(),
      revokeModelAccess: sinon.stub() }), true);
    const instance = renderer.getMountedInstance();
    instance.setState({ loadingUsers: true });
    const output = renderer.getRenderOutput();
    const spinner = output.props.children[2].props.children;
    const expected = React.createElement(
      'div',
      { className: 'sharing__loading' },
      React.createElement(juju.components.Spinner, null)
    );
    assert.deepEqual(spinner, expected);
  });

  it('can render with users', () => {
    const getModelUserInfo = sinon.stub().callsArgWith(0, null, users);
    const renderer = jsTestUtils.shallowRender(React.createElement(juju.components.Sharing, {
      canShareModel: false,
      getModelUserInfo: getModelUserInfo,
      grantModelAccess: sinon.stub(),
      humanizeTimestamp: humanizeTimestamp,
      revokeModelAccess: sinon.stub() }), true);
    const output = renderer.getRenderOutput();
    // Get all the children except the header, which is the first item in the
    // array.
    const obtained = output.props.children[2].props.children;
    const expected = [React.createElement(
      'div',
      { key: 'drwho@external', className: 'sharing__user' },
      React.createElement(
        'div',
        { className: 'sharing__user-details' },
        React.createElement(
          'div',
          { className: 'sharing__user-name' },
          'drwho'
        ),
        React.createElement(
          'div',
          { className: 'sharing__user-display-name' },
          'Ubuntu SSO',
          ' user'
        ),
        React.createElement(
          'div',
          { className: 'sharing__user-last-connection' },
          'last connection: 9 minutes ago'
        )
      ),
      React.createElement(
        'div',
        { className: 'sharing__user-access' },
        'admin'
      ),
      undefined
    ), React.createElement(
      'div',
      { key: 'rose', className: 'sharing__user' },
      React.createElement(
        'div',
        { className: 'sharing__user-details' },
        React.createElement(
          'div',
          { className: 'sharing__user-name' },
          'Rose'
        ),
        React.createElement(
          'div',
          { className: 'sharing__user-display-name' },
          'local',
          ' user'
        ),
        React.createElement(
          'div',
          { className: 'sharing__user-last-connection' },
          'never connected'
        )
      ),
      React.createElement(
        'div',
        { className: 'sharing__user-access' },
        'write'
      ),
      undefined
    ), React.createElement(
      'div',
      { key: 'dalek', className: 'sharing__user' },
      React.createElement(
        'div',
        { className: 'sharing__user-details' },
        React.createElement(
          'div',
          { className: 'sharing__user-name' },
          'Dalek'
        ),
        React.createElement(
          'div',
          { className: 'sharing__user-display-name' },
          'exterminate!'
        )
      )
    )];
    assert.deepEqual(obtained, expected);
  });

  it('can handle an API call error', () => {
    const getModelUserInfo = sinon.stub().callsArgWith(0, 'boom', []);
    const addNotification = sinon.stub();
    const closeHandler = sinon.stub();
    jsTestUtils.shallowRender(React.createElement(juju.components.Sharing, {
      addNotification: addNotification,
      canShareModel: false,
      closeHandler: closeHandler,
      getModelUserInfo: getModelUserInfo,
      grantModelAccess: sinon.stub(),
      revokeModelAccess: sinon.stub() }));
    assert.equal(addNotification.called, true);
    assert.deepEqual(addNotification.args[0][0], {
      title: 'Unable to load users.',
      message: 'Unable to load user information for this model: boom',
      level: 'error'
    });
    assert.equal(closeHandler.called, true);
  });

  it('can render the invitation form', () => {
    const grantModelAccess = sinon.stub();
    const renderer = jsTestUtils.shallowRender(React.createElement(juju.components.Sharing, {
      addNotification: sinon.stub(),
      canShareModel: true,
      closeHandler: sinon.stub(),
      getModelUserInfo: sinon.stub(),
      grantModelAccess: grantModelAccess,
      revokeModelAccess: sinon.stub() }), true);
    const output = renderer.getRenderOutput();
    const instance = renderer.getMountedInstance();
    const expectedOptions = [{
      label: 'Read',
      value: 'read'
    }, {
      label: 'Write',
      value: 'write'
    }, {
      label: 'Admin',
      value: 'admin'
    }];
    const expectedMarkup = React.createElement(
      'div',
      { className: 'sharing__invite' },
      React.createElement(
        'div',
        { className: 'sharing__invite--header' },
        'Add a user'
      ),
      React.createElement(
        'form',
        { onSubmit: instance._grantModelAccess },
        React.createElement(
          'div',
          { className: 'sharing__invite--username' },
          React.createElement(juju.components.GenericInput, {
            inlineErrorIcon: true,
            label: 'Username',
            placeholder: 'Username',
            onKeyUp: instance._handleUsernameInputChange,
            errors: false,
            ref: 'username',
            required: true })
        ),
        React.createElement(
          'div',
          { className: 'sharing__invite--access' },
          React.createElement(juju.components.InsetSelect, {
            label: 'Access',
            defaultValue: 'read',
            ref: 'access',
            options: expectedOptions })
        ),
        React.createElement(
          'div',
          { className: 'sharing__invite--grant-button' },
          React.createElement(juju.components.GenericButton, {
            submit: true,
            title: 'Add',
            tooltip: 'Add user',
            ref: 'grantButton',
            type: 'positive',
            disabled: true })
        ),
        undefined
      )
    );
    const actualMarkup = output.props.children[0];
    assert.deepEqual(actualMarkup, expectedMarkup);
  });

  it('can grant user access', () => {
    const grantModelAccess = sinon.stub();
    const renderer = jsTestUtils.shallowRender(React.createElement(juju.components.Sharing, {
      addNotification: sinon.stub(),
      canShareModel: true,
      closeHandler: sinon.stub(),
      getModelUserInfo: sinon.stub(),
      grantModelAccess: grantModelAccess,
      revokeModelAccess: sinon.stub() }), true);
    const output = renderer.getRenderOutput();
    const instance = renderer.getMountedInstance();
    // Verify that the action of granting access makes the expected API call.
    instance.refs = {
      username: { getValue: sinon.stub().returns('chekov') },
      access: { getValue: sinon.stub().returns('read') }
    };
    const inviteForm = output.props.children[0].props.children[1];
    inviteForm.props.onSubmit();
    assert.equal(grantModelAccess.called, true, 'grantModelAccess was not called');
    assert.deepEqual(grantModelAccess.args[0][0], 'chekov');
    assert.deepEqual(grantModelAccess.args[0][1], 'read');
    assert.isFunction(grantModelAccess.args[0][2]);
  });

  it('can render and revoke user access', () => {
    const getModelUserInfo = sinon.stub().callsArgWith(0, null, users);
    const revokeModelAccess = sinon.stub();
    const renderer = jsTestUtils.shallowRender(React.createElement(juju.components.Sharing, {
      addNotification: sinon.stub(),
      canShareModel: true,
      closeHandler: sinon.stub(),
      getModelUserInfo: getModelUserInfo,
      grantModelAccess: sinon.stub(),
      humanizeTimestamp: humanizeTimestamp,
      revokeModelAccess: revokeModelAccess }), true);
    const output = renderer.getRenderOutput();
    const instance = renderer.getMountedInstance();
    const user = output.props.children[2].props.children[0];
    // Validate that the markup is correct.
    const revokeMarkup = user.props.children[2];
    const revokeAction = revokeMarkup.props.children.props.action;
    const expectedMarkup = React.createElement(
      'div',
      { className: 'sharing__user-revoke' },
      React.createElement(juju.components.GenericButton, {
        action: revokeAction,
        tooltip: 'Remove user',
        icon: 'close_16' })
    );
    assert.deepEqual(revokeMarkup, expectedMarkup, 'markup for revoke button is incorrect');
    // Verify that the button triggers the API call as expected.
    revokeAction();
    assert.equal(revokeModelAccess.called, true, 'revokeModelAccess was not called');
    assert.deepEqual(revokeModelAccess.args[0], ['drwho@external', 'read', instance._modifyModelAccessCallback], 'revokeModelAccess not called with the correct data');
  });

  it('handles revoke/grant errors', () => {
    const renderer = jsTestUtils.shallowRender(React.createElement(juju.components.Sharing, {
      addNotification: sinon.stub(),
      canShareModel: true,
      closeHandler: sinon.stub(),
      getModelUserInfo: sinon.stub(),
      grantModelAccess: sinon.stub(),
      revokeModelAccess: sinon.stub() }), true);
    const instance = renderer.getMountedInstance();
    instance._modifyModelAccessCallback('boom');
    const output = renderer.getRenderOutput();
    const actualMessage = output.props.children[0].props.children[1].props.children[3];
    const expectedMessage = React.createElement(
      'div',
      { className: 'sharing__invite--error' },
      React.createElement(
        'b',
        null,
        'Error:'
      ),
      ' ',
      'boom'
    );
    assert.deepEqual(actualMessage, expectedMessage);
  });

  describe('add button states', () => {
    function makeSharingEle(state) {
      const renderer = jsTestUtils.shallowRender(React.createElement(juju.components.Sharing, {
        addNotification: sinon.stub(),
        canShareModel: true,
        closeHandler: sinon.stub(),
        getModelUserInfo: sinon.stub(),
        grantModelAccess: sinon.stub(),
        revokeModelAccess: sinon.stub() }), true);
      const instance = renderer.getMountedInstance();
      if (state) {
        instance.setState(state);
      }
      const output = renderer.getRenderOutput();
      return output.props.children[0].props.children[1].props.children[2].props.children;
    }
    it('shows a disabled button with "add" text by default', () => {
      const expected = React.createElement(juju.components.GenericButton, {
        submit: true,
        title: 'Add',
        tooltip: 'Add user',
        ref: 'grantButton',
        type: 'positive',
        disabled: true
      });
      assert.deepEqual(makeSharingEle(), expected);
    });

    it('shows an active button with "add" text when enabled', () => {
      const output = makeSharingEle({
        canAdd: true
      });
      const expected = React.createElement(juju.components.GenericButton, {
        submit: true,
        title: 'Add',
        tooltip: 'Add user',
        ref: 'grantButton',
        type: 'positive',
        disabled: false });
      assert.deepEqual(output, expected);
    });

    it('shows a disabled button with "add" text when sending', () => {
      const output = makeSharingEle({
        sending: true
      });
      const expected = React.createElement(juju.components.GenericButton, {
        submit: true,
        title: 'Add',
        tooltip: 'Add user',
        ref: 'grantButton',
        type: 'positive',
        disabled: true });
      assert.deepEqual(output, expected);
    });

    it('shows a disabled button with tick icon when sent', () => {
      const output = makeSharingEle({
        sent: true
      });
      const expected = React.createElement(juju.components.GenericButton, {
        submit: true,
        icon: 'tick_16',
        tooltip: 'Add user',
        ref: 'grantButton',
        type: 'positive',
        disabled: true });
      assert.deepEqual(output, expected);
    });
  });
});